/** 
*   Classe Date
*/

public class Date {

	private static final int daysInMonths[]={31,28,31,30,31,30,31,31,30,31,30,31};
	private static final String monthInFrench[]={ 
		"Janvier", "Février", "Mars",
		"Avril", "Mai", "Juin", "Juillet", "Aout",
		"Septembre", "Octobre", "Novembre", "Decembre"
		};
	private static final String dayInFrench[]={
	"Lundi", "Mardi", "Mercredi",
	"Jeudi", "Vendredi", "Samedi",
	"Dimanche"
	};
	
	int day;
	int month;
	int year;
	
	public Date(int day, int month, int year)
	{
		this.year = year;
		if(month<1 || month >12) 
			this.month=1;
		else 
			this.month=month;
		if(day<1 || day>daysInMonth()) 
			this.day=1;
		else 
			this.day=day;
	}
	
	/**
	 *  increments the date of one day
	 **/
	public void increment()
	{
		if(lastDayOfMonth()) {
			day = 1;
			if(month == 12){
				month = 1; 
				year++;
			} else {
				month++;
			}
		} else {
			day++;
		}
	}
	
	/**
	 *  @returns The number of days in the month this date belongs to.
	 **/
	private int daysInMonth()
	{
		return daysInMonths[month-1];
	}
	
	/**
	 * @returns true if this is the last day of the month.
	 */
	private boolean lastDayOfMonth()
	{
		return day == daysInMonth();
	}
	
	
	/**
	 * @returns the day number in the current year this date represents.
	 */	
	public int dayOfYear()
	{
		int dayOfYear=this.day;
		for(int i=0; i<month-1;i++)
			dayOfYear += daysInMonths[i];
		return dayOfYear;
	}
	
	/**
	* @returns 0=monday, 1=tuesday, ..., 6=sunday
	* see http://en.wikipedia.org/wiki/Zeller's_congruence
	 */	
	public int dayOfWeek()
	{
		int m = this.month; // local copies because
		int y = this.year;  // month and year can be modified
		if(m==1 || m==2)
		{
			m += 12;
			y--;            
		}
		
		int century = y/100;
		int yearOfCentury = y%100;
		int dayOfWeek = (day + 
						 + (((m+1)*26)/10)
						 + yearOfCentury 
						 + (yearOfCentury/4) 
						 + (century/4) 
						 + 5*century) % 7;
		
		return (dayOfWeek+5)%7;
	}
	
	
	public String toString()
	{
		return dayInFrench[dayOfWeek()]+" "+day+" "+ 
		       monthInFrench[month-1]+" "+year+ 
			   " le "+dayOfYear()+"-ième jour de l'année";
	}
	
	public static void main(String args[])
	{
		Date today = new Date(5, 11, 2009);
		System.out.println(today);
		
		Person prof = new Person("Hugues", "Bersini", new Date(6, 11, 1965));
		System.out.println(prof);
		
	}
}
